/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/** Hier koennen die Augenfarben abgerufen werden. */
public class Augenfarbe {

    /** Eine Liste aller Augenfarben. */
    private static final Vector< String > AUGENFARBEN = new Vector< String >();

    /** Eine Map in der unter jeder Augenfarbe ihre Id gespeichert ist. */
    private static final Map< String, String > AUGENFARBE_ID_MAP =
        new HashMap< String, String >();

    /** Eine Map in der jede Augenfarbe unter ihrer Id gespeichert ist. */
    private static final Map< String, String > ID_AUGENFARBE_MAP =
        new HashMap< String, String >();

    public static final String AMETHYST = I18n.i18n( "Amethyst" );
    public static final String BLAU = I18n.i18n( "blau" );
    public static final String BRAUN = I18n.i18n( "braun" );
    public static final String ROT = I18n.i18n( "rot" );
    public static final String VIOLETT = I18n.i18n( "violett" );

    static {
        final String[] ids = {
                "Amethyst",
                "bernsteinfarben",
                "blau",
                "braun",
                "dunkelbraun",
                "dunkelviolett",
                "grau",
                "graublau",
                "goldgesprenkelt",
                "gruen",
                "hellbraun",
                "rot",
                "saphirblau",
                "schwarz",
                "schwarzbraun",
                "smaragdgruen",
                "violett" };

        for ( final String id : ids ) {

            final String farbe = I18n.i18n( id );

            AUGENFARBEN.add( farbe );

            AUGENFARBE_ID_MAP.put( farbe, id );

            ID_AUGENFARBE_MAP.put( id, farbe );
        }

        Collections.sort( AUGENFARBEN, Util.COLLATOR );
    }

    /** @return eine alphabetisch sortierte Liste mit Augenfarben. */
    public static final Vector< String > getAugenfarben() {
        return new Vector< String>( AUGENFARBEN );
    }

    /**
     * @param haarfarbe
     *            deren Id gesucht ist.
     * @return die Id zur gegebenen Augenfarge.
     * @throws IllegalArgumentException
     *             wenn es zur uebergebenen Augenfarbe keine Id gibt.
     */
    public static final String getAugenfarbe( final String id ) {

        if ( ID_AUGENFARBE_MAP.containsKey( id ) ) {

            return ID_AUGENFARBE_MAP.get( id );

        } else {

            throw new IllegalArgumentException( "Es existiert keine " +
                    "Augenfarbe mit Id " + id );
        }
    }

    /**
     * @param augenfarbe
     *            deren Id gesucht ist.
     * @return die Id zur gegebenen Augenfarbe.
     * @throws IllegalArgumentException
     *             wenn es zur uebergebenen Augenfarbe keine Id gibt.
     */
    public static final String getId( final String augenfarbe ) {

        if ( AUGENFARBE_ID_MAP.containsKey( augenfarbe ) ) {

            return AUGENFARBE_ID_MAP.get( augenfarbe );

        } else {

            throw new IllegalArgumentException( "Es existiert keine " +
                    "Id zur Augenfarbe " + augenfarbe );
        }
    }

    /**
     * Prueft ob es zu einer Id einen passende Augenfarbe gibt.
     *
     * @param id zu der die Augenfarbe gesucht ist.
     * @return {@code true}, wenn es sich bei {@code id} um die Id einer
     *         Augenfarbe handelt, sonst {@code false}.
     */
    public static final boolean hatAugenfarbe( final String id ) {

        return ID_AUGENFARBE_MAP.containsKey( id );
    }

    /**
     * Prueft ob es zu einer Augenfarbe eine passende Id gibt.
     *
     * @param augenfarbe zu der die Id gesucht ist.
     * @return {@code true}, wenn es zu {@code augenfarbe} eine Id gibt, sonst
     *         {@code false}.
     */
    public static final boolean hatId( final String augenfarbe ) {

        return AUGENFARBE_ID_MAP.containsKey( augenfarbe );
    }
}
