/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/** Hier koennen die Haarfarben abgerufen werden. */
public class Haarfarbe {

    /** Eine Liste aller Haarfarben. */
    private static final Vector< String > HAARFARBEN = new Vector< String >();

    /** Eine Map in der unter jeder Haarfarbe ihre Id gespeichert ist. */
    private static final Map< String, String > HAARFARBE_ID_MAP =
        new HashMap< String, String >();

    /** Eine Map in der jede Haarfarbe unter ihrer Id gespeichert ist. */
    private static final Map< String, String > ID_HAARFARBE_MAP =
        new HashMap< String, String >();

    public static final String BLOND = I18n.i18n( "blond" );
    public static final String BRAUN = I18n.i18n( "braun" );
    public static final String WEISS = I18n.i18n( "weiss" );

    static {
        final String[] ids = {
                "blauschwarz",
                "blond",
                "braun",
                "dunkelblond",
                "dunkelbraun",
                "dunkelgrau",
                "feuerrot",
                "hellblond",
                "hellbraun",
                "hellgrau",
                "kupferrot",
                "mittelblond",
                "mittelbraun",
                "rot",
                "rotblond",
                "salzweiss",
                "schwarz",
                "silbern",
                "silberweiss",
                "weiss",
                "weissblond" };

        for ( final String id : ids ) {

            final String farbe = I18n.i18n( id );

            HAARFARBEN.add( farbe );

            HAARFARBE_ID_MAP.put( farbe, id );

            ID_HAARFARBE_MAP.put( id, farbe );
        }

        Collections.sort( HAARFARBEN, Util.COLLATOR );
    }

    /** @return eine alphabetisch sortierte Liste mit Haarfarben. */
    public static final Vector< String > getHaarfarben() {
        return new Vector< String>( HAARFARBEN );
    }

    /**
     * @param haarfarbe
     *            deren Id gesucht ist.
     * @return die Id zur gegebenen Haarfarbe.
     * @throws IllegalArgumentException
     *             wenn es zur uebergebenen Haarfarbe keine Id gibt.
     */
    public static final String getHaarfarbe( final String id ) {

        if ( ID_HAARFARBE_MAP.containsKey( id ) ) {

            return ID_HAARFARBE_MAP.get( id );

        } else {

            throw new IllegalArgumentException( "Es existiert keine " +
                    "Haarfarbe mit Id " + id );
        }
    }

    /**
     * @param haarfarbe
     *            deren Id gesucht ist.
     * @return die Id zur gegebenen Haarfarbe.
     * @throws IllegalArgumentException
     *             wenn es zur uebergebenen Haarfarbe keine Id gibt.
     */
    public static final String getId( final String haarfarbe ) {

        if ( HAARFARBE_ID_MAP.containsKey( haarfarbe ) ) {

            return HAARFARBE_ID_MAP.get( haarfarbe );

        } else {

            throw new IllegalArgumentException( "Es existiert keine " +
            		"Id zur Haarfarbe " + haarfarbe );
        }
    }

    /**
     * Prueft ob es zu einer Id einen passende Haarfarbe gibt.
     *
     * @param id zu der die Haarfarbe gesucht ist.
     * @return {@code true}, wenn es sich bei {@code id} um die Id einer
     *         Haarfarbe handelt, sonst {@code false}.
     */
    public static final boolean hatHaarfarbe( final String id ) {

        return ID_HAARFARBE_MAP.containsKey( id );
    }

    /**
     * Prueft ob es zu einer Haarfarbe eine passende Id gibt.
     *
     * @param haarfarbe zu der die Id gesucht ist.
     * @return {@code true}, wenn es zu {@code haarfarbe} eine Id gibt, sonst
     *         {@code false}.
     */
    public static final boolean hatId( final String haarfarbe ) {

        return HAARFARBE_ID_MAP.containsKey( haarfarbe );
    }
}
