/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;

/**
 * Eine Liste von Varianten die fest ist und nicht erweitert werden kann.
 *
 * @author Bernhard Jung
 */
class VariantenFest implements Varianten {

    /** Die Varianten die zur Verfuegung stehen. */
    private final List< Object > varianten;

    /**
     * @param dieVarianten die zur Verfuegung stehen sollen.
     */
    VariantenFest( final List< ? > dieVarianten ) {
        varianten = new ArrayList< Object >( dieVarianten );
    }

    /**
     * @throws UnsupportedOperationException wenn die Methode aufgerufen
     *      wird.
     */
    public final void einfuegen( final String variante ) {
        throw new UnsupportedOperationException();
    }

    /**
     *
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * @return eine Liste der Varianten die zur Verfuegung stehen.
     */
    public final List< Object > getVarianten(final Held held) {
        return new ArrayList< Object >( varianten );
    }

    /** @return {@code true} */
    public final boolean hatVarianten() {
        return true;
    }

    /** @return {@code false} */
    public final boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return getVarianten( null ).toString();
    }
}
