/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.gui.Util;

/**
 * Die Implementierung einer freien Variante, sie ist vom Benutzer erweiterbar.
 *
 * @author Bernhard Jung
 */
public class VariantenFrei implements Varianten {

    /** Die Varianten die zur Verfuegung stehen. */
    public final List< String > varianten;

    /** Erzeugt mit den uebergebenen Varianten eine neue Liste. */
    public VariantenFrei( final String ... dieVarianten ) {

        varianten = new ArrayList< String >( dieVarianten.length );

        for ( final String variante : dieVarianten ) {

            varianten.add( variante );
        }

        Collections.sort( varianten, Util.COLLATOR );
    }

    /** Erzeugt mit den uebergebenen Varianten eine neue Liste. */
    public VariantenFrei( final Collection< String > dieVarianten ) {

        varianten = new ArrayList< String >( dieVarianten );

        Collections.sort( varianten, Util.COLLATOR );
    }

    /** {@inheritDoc} */
    public void einfuegen( String variante ) {

        variante = variante.trim();

        int index = Collections.binarySearch( 
                varianten, 
                variante,
                Util.COLLATOR );

        if ( index < 0 ) {
            // Die Varianten ist noch nicht in der Liste.
            index = -( index + 1 );

            varianten.add( index, variante );
        }
    }

    /**
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * {@inheritDoc} */
    public List< Object > getVarianten(final Held held) {
        return new ArrayList< Object >( varianten );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code true} */
    public boolean istFreieVariante() {
        return true;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return "freie Varianten: " + getVarianten( null ).toString();
    }
}
