/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basiswerte;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

class Ausdauer extends Basiswert {

    Ausdauer(
            final String dieId,
            final int derIndex,
            final Formel dieFormel,
            final int derSteigerungsfaktor,
            final MaxZukauf derZukauf ) {
        super( dieId, derIndex, dieFormel, derSteigerungsfaktor, derZukauf );
    }

    /**
     * Halbiert die Stufe, wenn der Held Fettleibig ist.
     */
    @Override
    public int getStufe( final BasiswertWert wert ) {

        int stufe = super.getStufe( wert );

        if ( wert.getHeld().hat( VorNachteil.FETTLEIBIG ) ) {

            stufe = ( stufe + 1 ) / 2;
        }

        return stufe;
    }

    /**
     * Rechent Boni/Mali durch Ausdauernd/Kurzatmig ein.
     */
    @Override
    public int getModifikation( final Held held ) {

        int modifikation = super.getModifikation( held );

        if ( held.hat( VorNachteil.AUSDAUERND ) ) {

            final int x = held.getWert( VorNachteil.AUSDAUERND ).getStufe();

            modifikation += ( 2 * x );
        }

        if ( held.hat( VorNachteil.KURZATMIG ) ) {

            final int x = held.getWert( VorNachteil.KURZATMIG ).getStufe();

            modifikation -= ( 2 * x );
        }

        return modifikation;
    }
}
