/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Strings;

/**
 * Eine fest vorgegebene Talentliste die fuer jeden Helden immer die selben
 * Talente liefert.
 *
 * @author Bernhard Jung
 */
public class TalentListeFest implements TalentListe {

    /** Die Talente. */
    private final List< Talent > talente;

    /**
     * @param dieTalente die von dieser Liste geliefert werden sollen.
     * @throws NullPointerException wenn dieTalente {@code null} ist.
     */
    public TalentListeFest( final List< Talent > dieTalente ) {
        talente = new ArrayList< Talent >( dieTalente );

        Collections.sort( talente );
    }

    /**
     * @return eine Aufzaehlung wie 'Singen, Tanzen oder Zechen'. Enthaelt die
     *      Liste nur ein Talent, so wird der Name des Talents geliefert, also
     *      z.B. 'Schwerter'.
     */
    public final String getBeschreibung() {
        return Strings.erzeugeAufzaehlungMitOder( talente );
    }

    /** {@inheritDoc} */
    public final List<Talent> getTalente() {
        return new ArrayList< Talent >( talente );
    }

    /** {@inheritDoc} */
    public final List<Talent> getTalente(final Held held) {
        return new ArrayList< Talent >( talente );
    }

    /** {@inheritDoc} */
    public final Talentgruppe getTalentgruppe() {
        return talente.get( 0 ).getTalentgruppe();
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_TALENTLISTE );

        for ( final Talent talent : talente ) {

            final IXMLElement talentElement = new XMLElement( TAG_TALENT );

            talentElement.setContent( talent.getId() );

            element.addChild( talentElement );
        }

        return element;
    }
}
