/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.kosten;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_FEST;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_KOSTEN;

/**
 * Fuer Bausteine mit festen Kosten.
 *
 * @author Bernhard Jung
 */
public class KostenFest implements Kosten {

    private final int gpKosten;

    public KostenFest( final int dieKosten ) {
        gpKosten = dieKosten;
    }

    /** @return die festen GP-Kosten. */
    public final int getGpKosten() {
        return gpKosten;
    }

    /** {@inheritDoc} */
    public final int getGpKosten( final Held held ) {
        return gpKosten;
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement kosten = new XMLElement( TAG_KOSTEN );

        final IXMLElement fest = new XMLElement( TAG_FEST );

        fest.setContent( String.valueOf( gpKosten ) );

        kosten.addChild( fest );

        return kosten;
    }
}
