package de.bernhardjung.dsaprogramm.daten.factorys;

import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.ATTR_BEZEICHNUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.ATTR_SCHLUESSEL;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_MAENNLICH;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_MAENNLICH_ADLIG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_WEIBLICH;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_WEIBLICH_ADLIG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_NAMENLISTEN;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.basis.namen.Namenliste;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class FactoryNameFactorys extends Factory< NameFactory > {

    @Override
    public Collection< NameFactory > erzeugen( IXMLElement wurzelElement ) {
        
        final List< NameFactory > factorys = new ArrayList< NameFactory >();
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        
        for ( final IXMLElement element : wurzelElement.getChildren() )
        {
            final String id = element.getAttribute( ATTR_ID );
            
            final NameFactory factory = speicher.getNameFactory( id );
            factorys.add( factory );
            
            if ( element.hasAttribute( ATTR_BEZEICHNUNG ) ) {
                
                factory.setBezeichnung(  
                        element.getAttribute( ATTR_BEZEICHNUNG ) );
            }
            
            final IXMLElement formateElement =
                element.getFirstChildNamed( TAG_FORMATE );
            
            List< String > formate =  erzeugeFormateListe( 
                    formateElement, 
                    TAG_FORMATE_NAME_MAENNLICH );
                
            factory.setFormatStringsNameMaennlich( formate );
            
            formate = erzeugeFormateListe( 
                    formateElement, 
                    TAG_FORMATE_NAME_MAENNLICH_ADLIG );
            
            factory.setFormatStringsNameMaennlichAdlig( formate );
            
            formate = erzeugeFormateListe( 
                    formateElement, 
                    TAG_FORMATE_NAME_WEIBLICH );
            
            factory.setFormatStringsNameWeiblich( formate );
            
            formate = erzeugeFormateListe(
                    formateElement,
                    TAG_FORMATE_NAME_WEIBLICH_ADLIG );
            
            factory.setFormatStringsNameWeiblichAdlig( formate );
            
            factory.setNamenlisten( erzeugeNamenlisten( element ) );
        }
        
        return factorys;
    }

    /**
     * Erzeugt eine Liste von Formatvorlagen fuer Namen von Helden aus ihrer
     * XML-Repraesentation.
     * 
     * @param element
     *            Das XML-Element, das die Formate enthaelt. Die Formate sind
     *            eine Liste von Kindelementen eines Kindelements dieses
     *            Elements.
     * @param tagName
     *            Der name des Kindelements, dass die Liste der Formate
     *            enthaelt.
     * @return Die Liste der ermittelten Formate.
     */
    private List< String > erzeugeFormateListe( 
            final IXMLElement element,
            final String tagName) {
        
        final IXMLElement formateElement =
            element.getFirstChildNamed( tagName );
        
        final List< String > formate = new ArrayList< String >();
        
        for ( final IXMLElement formatElement : formateElement.getChildren() ) {
            
            formate.add( formatElement.getContent() );
        }
        
        return formate;
    }
    
    /**
     * Erzeugt eine Liste von Namenlisten aus ihrere XML-Repraesentation.
     * 
     * @param element dessen Kindelemente die Namenlisten sind.
     * @return die erzeugten Namenlisten.
     */
    private Map< String, Namenliste > erzeugeNamenlisten( 
            final IXMLElement element ) {
        
        final Map< String, Namenliste > namenlisten = 
            new HashMap< String, Namenliste >();

        final IXMLElement namenlistenElement = 
            element.getFirstChildNamed( TAG_NAMENLISTEN );
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        
        for ( final IXMLElement namenlisteElement : 
            namenlistenElement.getChildren() ) {
            
            final String id = namenlisteElement.getAttribute( ATTR_ID );
            
            final String schluessel = 
                namenlisteElement.getAttribute( ATTR_SCHLUESSEL );
            
            namenlisten.put( schluessel, speicher.getNamenliste( id ) );
        }
        
        return namenlisten;
    }    
}
