/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.JComboBox;
import javax.swing.MutableComboBoxModel;

/**
 * Ein ComboBoxModel fuer die Combobox, welche die Varianten eines
 * Vor-/Nachteils anzeigt. Das Model ist dazu gedacht die Varianten in der
 * Combobox schnell und einfach aktualisieren zu koennen. Der Versuch die
 * Varianten der Combobox mittels {@link JComboBox#addItem(Object)} zu
 * andern hatte eine lausige Laufzeit. Die Benutzeroberflaeche 'hing' fuer
 * mehrere Sekunden.
 *
 * @author Bernhard Jung
 */
public class VariantenListModel
extends AbstractListModel
implements MutableComboBoxModel {

    /** Das aktuell in der Combobox angezeigte Element. */
    private Object selectedItem;

    /** Die Elemente die die Comobbox zur Auswahl stellt. */
    private final List< Object > items;

    public VariantenListModel() {
        items = new ArrayList< Object >();
    }

    /** {@inheritDoc} */
    public final void addElement(final Object obj) {

        items.add( obj );

        final int index = items.size() - 1;

        fireIntervalAdded( this, index, index );
    }

    /** {@inheritDoc} */
    public final Object getSelectedItem() {
        return selectedItem;
    }

    /** {@inheritDoc} */
    public final void setSelectedItem(final Object anItem) {

        if ( ( null != anItem ) ) {

            if( ! anItem.equals( selectedItem ) ) {

                selectedItem = anItem;

                fireContentsChanged( this, -1, -1 );
            }

        } else if ( null != selectedItem ) {

            selectedItem = null;

            fireContentsChanged( this, -1, -1 );
        }
    }

    /** {@inheritDoc} */
    public final Object getElementAt(final int index) {
        return items.get( index );
    }

    /** {@inheritDoc} */
    public final int getSize() {
        return items.size();
    }

    /** {@inheritDoc} */
    public void insertElementAt( final Object obj, final int index ) {
        items.add( index, obj );

        fireIntervalAdded( this, index, index );
    }

    /** {@inheritDoc} */
    public void removeElement( final Object obj ) {

        final int index = items.indexOf( obj );

        if ( index >= 0 ) {

            items.remove( index );

            fireIntervalRemoved( this, index, index );
        }
    }

    /** {@inheritDoc} */
    public void removeElementAt( final int index ) {

        if ( getElementAt( index ) == selectedItem ) {
            if ( index == 0 ) {
                setSelectedItem( getSize() == 1 ? null : getElementAt( index + 1 ) );
            }
            else {
                setSelectedItem( getElementAt( index - 1 ) );
            }
        }

        items.remove( index );

        fireIntervalRemoved( this, index, index );
    }

    /**
     * Das ist die Methode auf die es ankommt. Hier koennen die in der
     * Combobox angezeigten Elemente in einem Rutsch aktualisert werden.
     *
     * @param newItems Die neuen Varianten fuer die ComboBox.
     */
    public final void setItems( final Collection< ? > newItems ) {

        items.clear();

        items.addAll( newItems );

        fireContentsChanged( this, 0, getSize() - 1 );

        if ( 0 < items.size() ) {

            setSelectedItem( items.get( 0 ) );

        } else {

            setSelectedItem( null );

        }

    }
}
