/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.KeyStroke;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.StdXMLReader;
import net.n3.nanoxml.XMLParserFactory;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.heldio.FactoryHeldXmlIn;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel.Warnen;
import de.bernhardjung.dsaprogramm.gui.steigern.PanelSteigern;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class ActionLadenHeldXml extends AbstractAction {

    /** Die Fehlermeldung wenn es ein Probelm beim Laden gibt. */
    private static final String FEHLER =
        I18n.i18n( "ActionLadenHeldXml.Fehler" );

    /** Der Name des Icons. */
    private static final String ICON = "laden";

    /** Der Name der Aktion, der im Menue angezeigt wird. */
    private static final String ACTION_NAME =
        I18n.i18n( "ActionLadenHeldXml.Name" );

    /** Die Taste um die Aktion im Menue auszloesen. */
    private static final String MNEMONIC =
        I18n.i18n( "ActionLadenHeldXml.Mnemonic" );

    /** Das Tastenkuerzel um die Aktion auszuloesen. */
    private static final String TASTEN =
        I18n.i18n( "ActionLadenHeldXml.Tasten" );

    /** Der Parser zum laden der Dateien. */
    private IXMLParser parser;

    /** Erzeugt den Helden aus dem XML-Dokument. */
    private final FactoryHeldXmlIn factory;

    /** Zum Auswaehlen der zu ladenden Datei. */
    private final JFileChooser fileChooser;

    /** Das Hauptfesnter der Anwendung. */
    private final Hauptpanel hauptpanel;

    public ActionLadenHeldXml(
            final Hauptpanel dasHauptpanel,
            final JFileChooser derFileChooser ) {

        super( ACTION_NAME, Util.ladeIcon( ICON ) );

        try {
            parser = XMLParserFactory.createDefaultXMLParser();
        } catch ( final Exception e ) {
            e.printStackTrace();
        }

        factory = new FactoryHeldXmlIn();

        fileChooser = derFileChooser;

        hauptpanel = dasHauptpanel;

        putValue( MNEMONIC_KEY, Util.getKeyCode( MNEMONIC ) );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( TASTEN ) );
    }

    public void actionPerformed( final ActionEvent e ) {

        final int laden = fileChooser.showDialog( hauptpanel, ACTION_NAME );

        if ( laden == JFileChooser.APPROVE_OPTION ) {

            final File datei = fileChooser.getSelectedFile();

            if ( datei.exists() ) {

                try {

                    final InputStream in = new FileInputStream( datei );

                    final IXMLReader reader = new StdXMLReader( in );

                    parser.setReader( reader );

                    final IXMLElement wurzelElement = (IXMLElement) parser.parse();

                    final Held held = factory.erzeugen( wurzelElement );

                    final PanelSteigern panelSteigern =
                        hauptpanel.getPanelSteigern();

                    panelSteigern.setHeld( held );

                    hauptpanel.anzeigen( 
                            panelSteigern,
                            Warnen.WARNEN );

                } catch (final Exception ex) {

                    DialogFehlermeldung.zeigeDialog( FEHLER, ex );

                }

            }
        }
    }

}
