/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class PanelMetaNameFactorys extends JPanel {

    /**
     * Der Titel fuer den Reiter der 1. Karteikarte auf der man die Angaben zur
     * Factory machen kann, wie Bezeichnung und Formate fuer die zu erzeugenden
     * Namen.
     */
    private static final String ALLGEMEIN_REITER =
        I18n.i18n( "PanelMetaNameFactorys.Allgemein.Reiter" );

    /**
     * Der Text vor dem Textfeld in das man die Bezeichnung der Factory eingeben
     * kann. 
     */
    private static final String BEZEICHNUNG =
        I18n.i18n( "PanelMetaNameFactorys.Bezeichnung" );

    /**
     * Der Text auf dem Knopf mit dem man die aktuell in der Liste gewaehlte
     * Factory loeschen kann.
     */
    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaNameFactorys.Entfernen" );

    /**
     * Der Text auf dem Reiter der Karteikarte auf der man die Formatvorlage
     * fuer die Namen der Helden waehlen kann.
     */
    private static final String FORMATE_REITER =
        I18n.i18n( "PanelMetaNameFactorys.Formate.Reiter" );

    /**
     * Die Meldung die dem Benutzer angezeigt wird, wenn er speichern will aber,
     * noch keine Datenquelle gewaehlt hat 
     */
    private static final String HINWEIS_DATENQUELLE =
        I18n.i18n( "PanelMetaNameFactorys.HinweisDatenquelle" );

    /**
     * Der Text vor dem Textfeld in das man die ID der Factory eingeben muss.
     */
    private static final String ID =
        I18n.i18n( "PanelMetaNameFactorys.Id" );

    /**
     * Der Text auf dem Reiter der Karteikarte auf der man die Formatvorlagen
     * fuer Namen von gemeinen/buergerlichen maennlichen Helden weaehlen kann.
     */
    private static final String MAENNLICH_REITER =
        I18n.i18n( "PanelMetaNameFactorys.Maennlich.Reiter" );

    /**
     * Der Text auf dem Reiter der Karteikarte auf der man die Formatvorlagen
     * fuer Namen von adligen maennlichen Helden waehlen kann.
     */
    private static final String MAENNLICH_ADLIG_REITER =
        I18n.i18n( "PanelMetaNameFactorys.MaennlichAdlig.Reiter" );

    /** Das Praefix fuer die ID einer NameFactory. */
    private static final String NAME_FACTORY_ID_PRAEFIX = "Namenformate.";
    
    /**
     * Der Text auf dem Reiter der 2. Karteikarte auf der man die Namenslisten
     * eingeben kann.
     */
    private static final String NAMENSLISTEN_REITER =
        I18n.i18n( "PanelMetaNameFactorys.Namenslisten.Reiter" );

    /**
     * Der Text auf dem Knopf mit dem man die eingegebene Factory in die Liste
     * der Factorys uebernimmt.
     */
    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaNameFactorys.Uebernehmen" );

    /**
     * Der Text auf dem Reiter der Karteikarte auf der man die Formatvorlagen
     * fuer Namen von gemeinen/buergelichen weiblichen Heldinnen waehlen kann.
     */
    private static final String WEIBLICH_REITER =
        I18n.i18n( "PanelMetaNameFactorys.Weiblich.Reiter" );

    /**
     * Der Text auf dem Reiter der Karteikarte auf der man die Formatvorlagen
     * fuer Namen von adligen weiblichen Heldinnen waehlen kann.
     */
    private static final String WEIBLICH_ADLIG_REITER =
        I18n.i18n( "PanelMetaNameFactorys.WeiblichAdlig.Reiter" );

    /** Das Textfeld, in das man die Bezeichnung der Factory eintragen kann. */
    private final JTextField bezeichnung;
    
    /** Die Liste, die die Factorys anzeigt. */
    private final JList factorys;

    /** 
     * Wenn der Benutzer die Factorys editiert wird im beim ersten mal ein
     * Hinweis angezeigt, dass er ein Verzeichnis als Datenquelle waehlen muss,
     * damit die Aenderungen gespeichert werden koennen. Nachdem der Hinweis
     * gezeigt wurd wird diese Variable von {@code false} auf {@code true}
     * gesetzt.
     */
    private boolean hinweisGezeigt;
    
    /** Das Textfeld, in das die ID der Factory eingetragen werden muss. */
    private final JTextField id;
    
    /**
     * Der Editor auf dem man die Formate fuer die namen von gemeinen,
     * maennlichen Helden waehlen kann. 
     */
    private final PanelMetaNameFormate maennlich;
    
    /**
     * Der Editor auf dem man die Formate fuer die Namen von adligen,
     * maennlichen Helden waehlen kann.
     */
    private final PanelMetaNameFormate maennlichAdlig;
    
    /**
     * Das Panel in dem man die Namenlisten der Factory bearbeiten kann.
     */
    private final PanelMetaNameFactorysNamenlisten namenlisten;
    
    /**
     * Der Button mit dem die Eingaben in den Editor fuer die Factory 
     * uebernommen werden.
     */
    private final JButton uebernehmen;
    
    /**
     * Der Editor auf dem man die Formatvorlagen fuer die Namen von gemeinen
     * weiblichen Helden waehlen kann.
     */
    private final PanelMetaNameFormate weiblich;
    
    /**
     * Der Editor mit dem man die Formate fuer die Namen von adligen weiblichen
     * Helden waehlen kann.
     */
    private final PanelMetaNameFormate weiblichAdlig;
    
    public PanelMetaNameFactorys() {

        bezeichnung = new JTextField();
        
        factorys = new JList();
        factorys.addListSelectionListener( new ListSelectionListener() {
            public void valueChanged( ListSelectionEvent e ) {
                anzeigen();
            }
        } );
        
        hinweisGezeigt = false;

        id = new JTextField();
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });
        
        maennlich = new PanelMetaNameFormate();
        
        maennlichAdlig = new PanelMetaNameFormate();

        namenlisten = new PanelMetaNameFactorysNamenlisten();

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                uebernehmen();
            }
        } );
        
        weiblich = new PanelMetaNameFormate();
        
        weiblichAdlig = new PanelMetaNameFormate();
        
        aufbauenBenutzeroberflaeche();
        
        setNameFactorys( Datenspeicher.getInstanz().getNameFactorys() );
        
        pruefen();
    }

    /**
     * Zeigt die aktuell in der Liste auf der linken Seite gewaehlte Factory
     * im Panel an.
     */
    private void anzeigen() {
        
        int index = factorys.getSelectedIndex();
        
        if ( 0 <= index ) {
            
            final NameFactory factory = 
                (NameFactory) factorys.getSelectedValue();
            
            bezeichnung.setText( factory.getBezeichnung() );
            
            id.setText( factory.getId().replace( NAME_FACTORY_ID_PRAEFIX, "" ));
            
            maennlich.setFormate(  factory.getFormatStringsNamenMaennlich() );
            
            maennlichAdlig.setFormate( 
                    factory.getFormatStringsNamenMaennlichAdlig() );
            
            namenlisten.setNamenlisten( factory.getNamenlisten() );
                
            weiblich.setFormate( factory.getFormatStringsNamenWeiblich() );
            
            weiblichAdlig.setFormate(
                    factory.getFormatStringsNamenWeiblichAdlig() );
        }
            
    }
    
    private void aufbauenBenutzeroberflaeche() {
        
        final String zeilen = "7dlu, F:P:G, 7dlu";
        
        final String spalten = "7dlu, F:100px:G(0.2), 4dlu, F:P:G(0.8), 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();

        // Die linke Seite zeigt eine Liste mit den Factories.
        add( erzeugeLinkeSpalte(), cc.xy( 2, 2 ) );
        
        // Auf der rechten Seite befindet sich das Tab in dem man die Daten
        // uber die Factory eingeben kann und wo man die Namensliste eingibt.
        add( erzeugeRechteSpalte(), cc.xy( 4, 2 ) );
    }

    /** 
     * Entfernt die aktuell in der Liste gewaehlte Factory aus der Liste der
     * Factories. 
     */
    private void entfernen() {
        
        final int index = factorys.getSelectedIndex();
        
        if ( 0 > index )
        {
            return;
        }
     
        final NameFactory factory = (NameFactory) factorys.getSelectedValue();
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        
        speicher.entfernen( factory );
        
        speichern();
        
        setNameFactorys( speicher.getNameFactorys() );
    }
    
    /**
     * Erzeugt die Liste der Factories.
     */
    private JComponent erzeugeLinkeSpalte() {
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        final JScrollPane scrollPane = new JScrollPane( factorys );
        
        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                entfernen();
            }
        } );
        
        panel.add( scrollPane, BorderLayout.CENTER );
        
        panel.add( entfernen, BorderLayout.PAGE_END );
        
        return panel;
    }
    
    /**
     * Erzeugt den Inhalt der ersten Karteikarte (Allgemein), auf der 
     * Id und Bezeichnung der Factory eingegeben werden kann, sowie die
     * Formate fuer die Namen.
     */
    private JComponent erzeugePanelAllgemein() {
        
        final FormLayout layout = new FormLayout( "P, 4dlu, F:P:G" );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        builder.setDefaultDialogBorder();
        
        builder.append( ID, id );
        
        builder.append( BEZEICHNUNG, bezeichnung );
        
        return builder.getPanel();
    }
    
    /**
     * @return Der Inhalt des zweiten Karteikarte auf der man die Formate fuer
     *         die Namen waehlen kann.
     */
    private JComponent erzeugePanelFormate() {
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        
        tabbedPane.addTab( MAENNLICH_REITER, maennlich );
        
        tabbedPane.addTab( MAENNLICH_ADLIG_REITER, maennlichAdlig );
        
        tabbedPane.addTab( WEIBLICH_REITER, weiblich );
        
        tabbedPane.addTab( WEIBLICH_ADLIG_REITER, weiblichAdlig );
        
        return tabbedPane;
    }
    
    /**
     * Erzeugt die Karteikarten auf der rechten Seite.
     */
    private JComponent erzeugeRechteSpalte() {
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        final JTabbedPane tabbedPane = new JTabbedPane();
        
        tabbedPane.addTab( ALLGEMEIN_REITER, erzeugePanelAllgemein() );
        
        tabbedPane.addTab( FORMATE_REITER, erzeugePanelFormate() );
        
        tabbedPane.addTab( NAMENSLISTEN_REITER, namenlisten );
        
        panel.add( tabbedPane, BorderLayout.CENTER );
        
        panel.add( uebernehmen, BorderLayout.PAGE_END );
        
        return panel;
    }

    /**
     * Prueft die Eingaben im Panel auf Korrektheit.
     *
     * @return {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( ! pruefen( id ) ) {
            
            ergebnis = false;
        }
        
        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }
    
    /**
     * Prueft ob in das Textfeld etwas eingegeben wurde.
     * 
     * @param textfeld das ueberprueft werden soll.
     * @return {@code true}, wenn in das Textfeld etwas eingegeben wurde, sonst
     *         {@code false}.
     */
    private boolean pruefen( final JTextField textfeld ) {
        
        final boolean ergebnis;
        
        if ( textfeld.getText().trim().equals( "" ) ) {

            textfeld.setBackground( Util.FEHLERFARBE );
            
            ergebnis = false;

        } else {

            textfeld.setBackground( bezeichnung.getBackground() );
            
            ergebnis = true;
        }
        
        return ergebnis;
    }

    /** Loescht alle Eingaben aus dem Editor. */
    private void reset()
    {
        bezeichnung.setText( "" );
        
        id.setText( "" );
        
        maennlich.reset();
        
        maennlichAdlig.reset();
        
        namenlisten.reset();
        
        weiblich.reset();
        
        weiblichAdlig.reset();
    }
    
    private void setNameFactorys( final List< NameFactory > liste ) {
        
        final Vector< NameFactory > vector = new Vector< NameFactory >( liste );
        
        factorys.setListData( vector );
    }

    /**
     * Speichert die Factorys auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereNameFactorys();

        } else if ( ! hinweisGezeigt ) {

            JOptionPane.showMessageDialog( this, HINWEIS_DATENQUELLE );
            
            hinweisGezeigt = true;
        }
    }

    private void uebernehmen() {
        
        if ( ! pruefen()  ) {
            
            return;
        }
        
        final String id = NAME_FACTORY_ID_PRAEFIX + this.id.getText().trim();
        
        Datenspeicher speicher = Datenspeicher.getInstanz();
        
        final NameFactory factory = speicher.getNameFactory( id );
        
        factory.setBezeichnung( bezeichnung.getText().trim() );
        
        factory.setFormatStringsNameMaennlich( maennlich.getFormate() );
        
        factory.setFormatStringsNameMaennlichAdlig(
                maennlichAdlig.getFormate() );
        
        factory.setFormatStringsNameWeiblich( weiblich.getFormate() );
        
        factory.setFormatStringsNameWeiblichAdlig(
                weiblichAdlig.getFormate() );
        
        factory.setNamenlisten( namenlisten.getNamenlisten() );
        
        speicher.einfuegen( factory );
        
        speichern();
        
        setNameFactorys( speicher.getNameFactorys() );
        
        reset();
    }
}
