/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.Name;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Ein Editor mit dem sich eine Liste von Namen anzeigen und bearbeiten laesst.
 * Er besitzt eine List mit Namen auf der linken Seite und rechts davon
 * zwei Textfelder, mit jeweils einem Hinzufuegen-Knopf. In eines der
 * Textfelder kann man einen Schluessel aus der Lokalisierungsdatei eingeben.
 * Ueber diesen wird dann beim Klick auf hinzufuegen der Name geladen.
 * Das andere Textfeld erlaubt es einen Namen direkt einzugeben. 
 */
class PanelMetaNamenliste extends JPanel {

    private static final String PREFIX_ID = "Name.";
    
    /**
     * Der Text auf dem Knopf mit dem man einen eingegebenen Namen aus der Liste
     * von Namen entfernt.
     */
    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaNamenliste.Entfernen" );

    /**
     * Der Text auf den Knoepfen mit denen man einen eingegebenen Namen oder
     * Lokalisierungschluessel zu der Liste von Namen hinzufuegt.
     */
    private static final String EINFUEGEN =
        I18n.i18n( "PanelMetaNamenliste.Einfuegen" );

    /**
     * Der Text fuer den Separator ueber dem Textfeld fuer den 
     * Lokalisierungsschluessel.
     */
    private static final String LOKALISIERUNG_SEPARATOR =
        I18n.i18n( "PanelMetaNamenliste.Lokalisierung.Separator" );

    /**
     * Der Text fuer den Separator ueber dem Textfeld in dem ein Namen 
     * eingegeben werden kann.
     */
    private static final String NAME_SEPARATOR =
        I18n.i18n( "PanelMetaNamenliste.Name.Separator" );

    /** 
     * Das Textfeld in das ein Schluessel aus der Lokalisierungsdatei eingegeben
     * werden kann.
     */
    private JTextField lokalisierung;
    
    /**
     * Das Textfeld in das ein Name direkt eingegeben werden kann.
     */
    private JTextField name;
    
    /** Die Liste der Namen die in dem Panel bearbeitet wird. */
    private NamenListModel namen;
    
    public PanelMetaNamenliste() {
    
        lokalisierung = new JTextField();
        lokalisierung.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenLokalisierung();
            }
        } );
        
        name = new JTextField();
        name.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenName();
            }
        } );
        
        namen = new NamenListModel();
        
        aufbauenBenutzeroberflaeche();
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     */
    private void aufbauenBenutzeroberflaeche() {
        
        final String zeilen = "F:P:G";
        
        final String spalten = "0dlu, F:100px:G(0.2), 4dlu, F:P:G(0.8)";

        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();
        
        add( erzeugeLinkeSpalte(), cc.xy( 2, 1 ) );
        
        add( erzeugeRechteSpalte(), cc.xy( 4, 1 ) );
    }
    
    /**
     * @return Die rechte spalte des Editors mit den Textfeldern.
     */
    private JComponent erzeugeRechteSpalte() {
        
        lokalisierung = new JTextField();
        
        final JButton lokalisierungHinzufuegen = new JButton( EINFUEGEN );
        lokalisierungHinzufuegen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenLokalisierung();
            }
        });
        
        name = new JTextField();
        
        final JButton nameHinzufuegen = new JButton( EINFUEGEN );
        nameHinzufuegen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenName();
            }
        });
        final String zeilen = "P, 4dlu, P, 4dlu, P, 7dlu, " +
        		              "P, 4dlu, P, 4dlu, P, 0px:G";
        
        final String spalten = "F:P:G";
        
        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        
        builder.appendSeparator( LOKALISIERUNG_SEPARATOR );
        builder.nextLine( 2 );
        builder.append( lokalisierung );
        builder.nextLine( 2 );
        builder.append( lokalisierungHinzufuegen );
        
        builder.nextLine( 2 );
        builder.appendSeparator( NAME_SEPARATOR );
        builder.nextLine( 2 );
        builder.append( name );
        builder.nextLine( 2 );
        builder.append( nameHinzufuegen );
        
        return builder.getPanel();
    }

    /**
     * @return Die linke Spalte des Editors mit der Liste der Namen.
     */
    private JComponent erzeugeLinkeSpalte() {
        
        final JPanel panel = new JPanel( new BorderLayout() );
        
        final JList liste = new JList( namen );

        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                namen.entfernen( liste.getSelectedIndex() );
            }
        } );
        
        final JScrollPane scrollPane = new JScrollPane( liste );
        
        panel.add( scrollPane, BorderLayout.CENTER );
        
        panel.add( entfernen, BorderLayout.PAGE_END );
        
        return panel;
    }
    
    /**
     * @return Die Liste der eingegebenen Namen.
     */
    public List< Name > getNamen() {
        
        return namen.getNamen();
    }
    
    /**
     * Fuegt der Liste der Namen einen neuen Namen hinzu. Der Name wird ueber
     * den Schluessel, der in das Textfeld {@link #lokalisierung} eingegeben
     * wurde initialisiert.
     */
    private void hinzufuegenLokalisierung() {
        
        String schluessel = lokalisierung.getText().trim();
        
        if ( schluessel.equals( "" ) ) {
            
            return;
        }
        
        schluessel = PREFIX_ID + schluessel;
        
        final Name name = new Name();
        
        name.setLokalisierungsschluessel( schluessel );
        
        namen.hinzufuegen( name );
    }
    
    /**
     * Fuegt der Liste der Namen einen neuen Namen hinzu. Als Name wird der
     * Text der in das Textfeld {@link #name} eingegeben wurde genutzt.
     */
    private void hinzufuegenName() {

        final String nameString = name.getText().trim();
        
        if ( nameString.equals( "" ) ) {
            
            return;
        }
        
        final Name name = new Name();
        
        name.setName( nameString );
        
        namen.hinzufuegen( name );
    }
    
    /**
     * @param namenliste Die Namen die im Editor angezeigt und bearbeitet werden
     *          sollen.
     */
    public void setNamen( final List< Name > namenliste ) {
        
        this.namen.setNamen( namenliste );
    }
    
    /**
     * Das Model fuer die Liste der Namen auf der linken Seite.
     */
    private class NamenListModel extends AbstractListModel {

        private final List< Name > namen = new ArrayList< Name >();
        
        /**
         * Sortiert einen Namen in die Liste ein. Wenn der Namen schon in der
         * Liste ist wird nichts gemacht.
         * 
         * @param name Ein neuer Name.
         */
        public void hinzufuegen( final Name name ) {
            
            int index = Collections.binarySearch( namen, name );
            
            if ( 0 > index )  {
                
                index = - ( index + 1 );
                
                namen.add( index, name );
                
                fireIntervalAdded( this, index, index );
            }
        }
        
        /**
         * Enternt einen Namen von der angegebenen Position der Liste.
         * 
         * @param index Der index des zu entfernenden Namens.
         */
        public void entfernen( int index ) {
            
            if ( ( index < 0 ) || ( namen.size() <= index ) ) {
                
                return;
            }
                
            namen.remove( index );
            
            fireIntervalRemoved( this, index, index );
        }
        
        /**
         * Liefert den Namen von der angegebenen Position der Liste.
         */
        public Object getElementAt( int index ) {
            
            return namen.get( index );
        }

        /**
         * @return Die Namen im Model.
         */
        public List< Name > getNamen() {
            
            return new ArrayList< Name >( namen );
        }
        
        /**
         * Liefert die Anzahl von Namen in der Liste.
         */
        public int getSize() {
            
            return namen.size();
        }
     
        /**
         * @param neueNamen Die Namen fuer die Liste.
         */
        public void setNamen( final List< Name > neueNamen ) {
            
            namen.clear();
            
            namen.addAll( neueNamen );
            
            fireContentsChanged( this, 0, -1 + namen.size() );
        }
    }
}
