/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.action.ActionLadenRasse;
import de.bernhardjung.dsaprogramm.gui.action.ActionSpeichernRasse;
import de.bernhardjung.dsaprogramm.gui.meta.PanelMetaWuerfeltabelle.ElementQuelle;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelMetaRasse extends JPanel {

    private static final String AUGEN_FRAME =
        I18n.i18n( "PanelMetaRasse.Augenfarben" );

    private static final String AUGEN_REITER =
        I18n.i18n( "PanelMetaRasse.Augenfarben.Reiter" );

    private static final String DIVERSES_FRAME =
        I18n.i18n( "PanelMetaRasse.Diverses" );

    private static final String DIVERSES_REITER =
        I18n.i18n( "PanelMetaRasse.Diverses.Reiter" );

    private static final String EMPF_VOR_NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaRasse.EmpfohleneVorNachteile" );

    private static final String EMPF_VOR_NACHTEILE_REITER =
        I18n.i18n( "PanelMetaRasse.EmpfohleneVorNachteile.Reiter" );

    private static final String HAARE_FRAME =
        I18n.i18n( "PanelMetaRasse.Haarfarben" );

    private static final String HAARE_REITER =
        I18n.i18n( "PanelMetaRasse.Haarfarben.Reiter" );

    private static final String KULTUREN_FRAME =
        I18n.i18n( "PanelMetaRasse.Kulturen" );

    private static final String KULTUREN_REITER =
        I18n.i18n( "PanelMetaRasse.Kulturen.Reiter" );

    private static final String LEITTALENTE_FRAME =
        I18n.i18n( "PanelMetaRasse.Leittalente" );

    private static final String LEITTALENTE_REITER =
        I18n.i18n( "PanelMetaRasse.Leittalente.Reiter" );

    private static final String MODIFIKATIONEN_FRAME =
        I18n.i18n( "PanelMetaRasse.Modifikationen" );

    private static final String MODIFIKATIONEN_REITER =
        I18n.i18n( "PanelMetaRasse.Modifikationen.Reiter" );

    private static final String NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaRasse.Nachteile" );

    private static final String NACHTEILE_REITER =
        I18n.i18n( "PanelMetaRasse.Nachteile.Reiter" );

    private static final String NEU =
        I18n.i18n( "PanelMetaRasse.Neu" );

    private static final String TALENTE_FRAME =
        I18n.i18n( "PanelMetaRasse.Talente" );

    private static final String TALENTE_REITER =
        I18n.i18n( "PanelMetaRasse.Talente.Reiter" );

    private static final String UNGE_VOR_NACHTEILE_FRAME =
        I18n.i18n( "PanelMetaRasse.UngeeigneteVorNachteile" );

    private static final String UNGE_VOR_NACHTEILE_REITER =
        I18n.i18n( "PanelMetaRasse.UngeeigneteVorNachteile.Reiter" );

    private static final String VORTEILE_FRAME =
        I18n.i18n( "PanelMetaRasse.Vorteile" );

    private static final String VORTEILE_REITER =
        I18n.i18n( "PanelMetaRasse.Vorteile.Reiter" );

    private final PanelMetaWuerfeltabelle augenfarben;

    private final PanelMetaRasseDiverses diverses;

    private final PanelMetaListe< VorNachteil > empfohlene;

    private final PanelMetaWuerfeltabelle haarfarben;

    private final PanelMetaListeKulturen kulturen;

    private final PanelMetaLeittalente leittalente;

    private final PanelMetaModifikationen modifikationen;

    private final PanelMetaVorNachteile nachteile;

    private final PanelMetaTalentboni talentboni;

    private final PanelMetaUngeeigneteVorNachteile ungeeignete;

    private final PanelMetaVorNachteile vorteile;

    public PanelMetaRasse(
            final JFrame hauptfenster,
            final Hauptpanel hauptpanel ) {

        augenfarben = new PanelMetaWuerfeltabelle( new ElementQuelle(){
            public Vector< String > getElemente() {
                return Augenfarbe.getAugenfarben();
            }
            public String getId( final String augenfarbe ) {
                if ( Augenfarbe.hatId( augenfarbe ) ) {
                    return Augenfarbe.getId( augenfarbe );
                }
                return augenfarbe;
            }
        });

        diverses = new PanelMetaRasseDiverses( this );

        empfohlene = new PanelMetaListe< VorNachteil>(
                VorNachteil.getVorNachteile() );

        haarfarben = new PanelMetaWuerfeltabelle( new ElementQuelle(){
            public Vector< String > getElemente() {
                return Haarfarbe.getHaarfarben();
            }
            public String getId( final String haarfarbe ) {
                if ( Haarfarbe.hatId( haarfarbe ) ) {
                    return Haarfarbe.getId( haarfarbe );
                }
                return haarfarbe;
            }
        });

        kulturen = new PanelMetaListeKulturen();

        leittalente = new PanelMetaLeittalente();

        modifikationen = new PanelMetaModifikationen(
                Arrays.asList( Basiswert.LE, Basiswert.AU, Basiswert.MR ) );

        nachteile = new PanelMetaVorNachteile( VorNachteil.getNachteile(),
                                               NACHTEILE_REITER );

        talentboni = new PanelMetaTalentboni(
                PanelMetaTalentboni.Modus.RASSE );

        ungeeignete = new PanelMetaUngeeigneteVorNachteile();

        vorteile = new PanelMetaVorNachteile( VorNachteil.getVorteile(),
                                              VORTEILE_REITER );

        // Button Bar
        // **********
        final JButton neu = new JButton( NEU );
        neu.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                neu();
            }
        });

        final JPanel buttonBar = new JPanel(
                new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );

        final CellConstraints cc = new CellConstraints();

        buttonBar.add( ButtonBarFactory.buildLeftAlignedBar(
                    new JButton( new ActionLadenRasse( this ) ),
                    new JButton( new ActionSpeichernRasse(
                                        hauptfenster, hauptpanel ) ),
                    neu ),
                    cc.xy( 2, 2 ) );

        // Tabbed Pane
        // ***********
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        tabbedPane.addTab( DIVERSES_REITER,
                           erzeugeFrame( DIVERSES_FRAME, diverses ) );

        tabbedPane.addTab( MODIFIKATIONEN_REITER,
                           erzeugeFrame( MODIFIKATIONEN_FRAME,
                                         Util.erzeugeScrollPane( modifikationen ) ) );

        tabbedPane.addTab( AUGEN_REITER,
                           erzeugeFrame( AUGEN_FRAME,
                                         Util.erzeugeScrollPane( augenfarben ) ) );

        tabbedPane.addTab( HAARE_REITER,
                           erzeugeFrame( HAARE_FRAME,
                                         Util.erzeugeScrollPane( haarfarben ) ) );

        tabbedPane.addTab( KULTUREN_REITER,
                           erzeugeFrame( KULTUREN_FRAME,
                                         Util.erzeugeScrollPane( kulturen ) ) );

        tabbedPane.addTab( TALENTE_REITER,
                           erzeugeFrame( TALENTE_FRAME, talentboni ) );

        tabbedPane.addTab( LEITTALENTE_REITER,
                           erzeugeFrame( LEITTALENTE_FRAME,
                                         Util.erzeugeScrollPane( leittalente ) ) );

        tabbedPane.addTab( VORTEILE_REITER,
                           erzeugeFrame( VORTEILE_FRAME, vorteile ) );

        tabbedPane.addTab( NACHTEILE_REITER,
                           erzeugeFrame( NACHTEILE_FRAME, nachteile ) );

        tabbedPane.addTab( EMPF_VOR_NACHTEILE_REITER,
                           erzeugeFrame( EMPF_VOR_NACHTEILE_FRAME,
                                         Util.erzeugeScrollPane( empfohlene ) ) );

        tabbedPane.addTab( UNGE_VOR_NACHTEILE_REITER,
                           erzeugeFrame( UNGE_VOR_NACHTEILE_FRAME,
                                         Util.erzeugeScrollPane( ungeeignete ) ) );

        setLayout( new BorderLayout() );

        add( buttonBar, BorderLayout.PAGE_START  );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Rasse rasse ) {

        augenfarben.anzeigen( rasse.getAugenfarben() );

        diverses.anzeigen( rasse );

        empfohlene.setGewaehlteElemente( rasse.getEmpfohleneVorNachteile() );

        haarfarben.anzeigen( rasse.getHaarfarben() );

        kulturen.setGewaehlteElemente( rasse.getKulturen() );

        leittalente.anzeigen( rasse );

        modifikationen.anzeigen( rasse );

        nachteile.anzeigen( rasse.getNachteile() );

        talentboni.anzeigen( rasse );

        ungeeignete.anzeigen( rasse.getUngeeigneteVorNachteile() );

        vorteile.anzeigen( rasse.getVorteile() );
    }

    private SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JComponent inhalt ) {

        final SimpleInternalFrame frame =
            new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( inhalt );

        return frame;
    }

    /** @return Die Id, die der Benutzer fuer die Rasse angegeben hat. */
    public String getRassenId() {
        return diverses.getRassenId();
    }

    /** @return den Namen, den der Benutzer fuer die Rasse angegeben hat. */
    public String getRassenName() {
        return diverses.getRassenName();
    }

    /**
     * Setzt das Panel wieder zurueck, damit man eine neue Kultur
     * eingeben kann.
     */
    private void neu() {

        diverses.reset();

        empfohlene.reset();

        kulturen.reset();

        leittalente.reset();

        modifikationen.reset();

        nachteile.reset();

        talentboni.reset();

        ungeeignete.reset();

        vorteile.reset();
    }

    public final void uebertrageDaten( final Rasse rasse ) {

        rasse.setAugenfarben( augenfarben.getWuerfeltabelle() );

        diverses.uebertrageDaten( rasse );

        rasse.setEmpfohlene( empfohlene.getGewaehlteElemente() );

        rasse.setHaarfarben( haarfarben.getWuerfeltabelle() );

        rasse.setKulturen( kulturen.getGewaehlteElemente() );

        leittalente.uebertrageDaten( rasse );

        modifikationen.uebertrageDaten( rasse );

        rasse.setNachteile( nachteile.getBoni() );

        rasse.setTalente( talentboni.getBoni() );

        rasse.setUngeeignete( ungeeignete.getGewaehlteVorNachteile() );

        rasse.setVorteile( vorteile.getBoni() );
    }
}
