/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

public class PanelMetaRuestungen extends JPanel {

    private static final String BE =
        I18n.i18n( "PanelMetaRuestungen.Behinderung" );

    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaRuestungen.Entfernen" );

    private static final String HINWEIS_DATENQUELLE =
        I18n.i18n( "PanelMetaRuestungen.HinweisDatenquelle" );

    private static final String ID =
        I18n.i18n( "PanelMetaRuestungen.Id" );

    private static final String MINDEST_TORSO_RS =
        I18n.i18n( "PanelMetaRuestungen.MindestTorsoRs");

    private static final String NAME =
        I18n.i18n( "PanelMetaRuestungen.Name" );

    private static final String RS =
        I18n.i18n( "PanelMetaRuestungen.Ruestungsschutz" );

    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaRuestungen.Uebernehmen");

    private static final String ZUSATZRUESTUNG =
        I18n.i18n( "PanelMetaRuestungen.Zusatzruestung");

    /** Die Behinderung. */
    private final JSpinner be;

    /**
     * Gibt an ob der Benutzer schon darauf hingewiesen wurde eine Datenquelle
     * zu waehlen.
     */
    private boolean hinweisGezeigt;

    /** Die Id. */
    private final JTextField id;

    /** Der mindest RS auf den Torso fuer eine Zusatzruestung. */
    private final JSpinner minTorsoRs;

    private final JLabel minTorsoRsLabel;

    /** Der Name. */
    private final JTextField name;

    /** Der Ruestungsschutz. */
    private final JSpinner rs;

    /** Die Liste der Ruestungen. */
    private final JList ruestungen;

    /** Da sind die Ruestungen gespeichert. */
    private final Datenspeicher speicher;

    /**
     * Der Knopf mit dem die eingegebene Ruestung in die Liste der Ruestungen
     * uebernommen wird.
     */
    private final JButton uebernehmen;

    /** Gib an ob es such um eine Zusatzruestung handelt.*/
    private final JCheckBox zusatzruestung;

    public PanelMetaRuestungen() {

        be = Util.erzeugeSpinner( 0, 0, 99, 1 );

        hinweisGezeigt = false;

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        id = new JTextField();
        id.setBackground( Util.FEHLERFARBE );
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });

        minTorsoRs = Util.erzeugeSpinner( 0, 0, 99, 1 );
        minTorsoRs.setEnabled( false );

        minTorsoRsLabel = new JLabel( MINDEST_TORSO_RS );
        minTorsoRsLabel.setEnabled( false );

        name = new JTextField();

        rs = Util.erzeugeSpinner( 0, 0, 99, 1 );

        speicher = Datenspeicher.getInstanz();

        final Vector< Ruestung > vector =
            new Vector< Ruestung >( speicher.getRuestungen() );

        ruestungen = new JList( vector );
        ruestungen.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        ruestungen.addListSelectionListener( new ListSelectionListener(){
            public void valueChanged( final ListSelectionEvent e ) {

                if ( e.getValueIsAdjusting() ) {
                    return;
                }

                anzeigen();
            }
        });

        zusatzruestung = new JCheckBox( ZUSATZRUESTUNG );
        zusatzruestung.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                if ( zusatzruestung.isSelected() ) {
                    minTorsoRsLabel.setEnabled( true );
                    minTorsoRs.setEnabled( true );
                } else {
                    minTorsoRsLabel.setEnabled( false );
                    minTorsoRs.setEnabled( false );
                    minTorsoRs.setValue( 0 );
                }
            }
        });

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die Werte der in der Liste gewaehlte Ruestung an.
     */
    private final void anzeigen() {

        final int index = ruestungen.getSelectedIndex();

        if ( 0 <= index ) {

            final Ruestung ruestung = (Ruestung) ruestungen.getSelectedValue();

            be.setValue( ruestung.getBe() );

            id.setText( ruestung.getId().replace( "Ruestung.", "" ) );

            minTorsoRs.setValue( ruestung.getMindestTorsoRs() );

            name.setText( ruestung.getName() );

            rs.setValue( ruestung.getRs() );

            if ( ruestung.istZusatzruestung() ) {

                minTorsoRsLabel.setEnabled( true );
                minTorsoRs.setEnabled( true );
                zusatzruestung.setSelected( true );

            } else {

                minTorsoRsLabel.setEnabled( false );
                minTorsoRs.setEnabled( false );
                zusatzruestung.setSelected( false );
            }
        }
    }

    private final void aufbauenBenutzeroberflaeche() {

        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                entfernen();
            }
        });

        final JPanel buttonpanel =
            ButtonBarFactory.buildLeftAlignedBar( uebernehmen );

        final String spalten = "7dlu, 200px:G, 4dlu, P, 4dlu, P, P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, " +
            "4dlu, P, 4dlu, P, P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( ruestungen ), cc.xywh( 2, 2, 1, 14 ) );

        add( entfernen, cc.xy( 2, 17 ) );

        int y = 0;

        add( new JLabel( NAME ), cc.xy ( 4, y+=2 ) );
        add( name,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( ID ), cc.xy ( 4, y+=2 ) );
        add( id,               cc.xyw( 6, y, 2 ) );

        add( new JLabel( RS ), cc.xy( 4, y+=2 ) );
        add( rs,               cc.xy( 6, y ) );

        add( new JLabel( BE ), cc.xy( 4, y+=2 ) );
        add( be,               cc.xy( 6, y ) );

        add( zusatzruestung, cc.xyw( 4, y+=2, 4 ) );

        add( minTorsoRsLabel, cc.xy( 4, y+=2 ) );
        add( minTorsoRs,      cc.xy( 6, y ) );

        add( buttonpanel, cc.xyw( 4, y+=2, 4 ) );
    }

    /**
     * Enternt die in der Liste {@link #ruestungen} gewaehlte Ruestung aus dem
     * Datenspeicher.
     */
    private void entfernen() {

        final int index = ruestungen.getSelectedIndex();

        if ( 0 <= index ) {

            final Ruestung ruestung =
                (Ruestung) ruestungen.getSelectedValue();

            speicher.entfernen( ruestung );

            final List< Ruestung > liste = speicher.getRuestungen();

            final Vector< Ruestung > vector =
                new Vector< Ruestung >( liste );

            ruestungen.setListData( vector );

            speichern();
        }
    }

    /**
     * Prueft die Eingabe im Panel auf Korrektheit und setzt den
     * Uebernehmen-Knopf auf aktiv oder deaktiv.
     *
     * @return  {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        if ( id.getText().trim().equals( "" ) ) {

            id.setBackground( Util.FEHLERFARBE );

            uebernehmen.setEnabled( false );

            return false;

        } else {

            id.setBackground( name.getBackground() );

            uebernehmen.setEnabled( true );

            return true;
        }
    }

    /**
     * Speichert die Liste der Ruestungen auf die Festplatte.
     */
    private final void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereRuestungen();

        } else if ( ! hinweisGezeigt ){

            JOptionPane.showMessageDialog( this, HINWEIS_DATENQUELLE );

            hinweisGezeigt = true;
        }

    }

    /**
     * Uebernimmt die im Panel eingegebenen Ruestung in die Liste der
     * Ruestungen.
     */
    private final void uebernehmen() {

        String idString = id.getText().trim();

        idString = "Ruestung." + idString;

        final Ruestung ruestung = speicher.getRuestung( idString );

        ruestung.setBe( (Integer) be.getValue() );

        ruestung.setMindestTorsoRs( (Integer) minTorsoRs.getValue() );

        ruestung.setName( name.getText().trim() );

        ruestung.setRs( (Integer) rs.getValue() );

        ruestung.setZusatzruestung( zusatzruestung.isSelected() );

        speicher.einfuegen( ruestung );

        speichern();

        final List< Ruestung > liste = speicher.getRuestungen();

        final Vector< Ruestung > vector = new Vector< Ruestung >( liste );

        ruestungen.setListData( vector );
    }
}
