/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListe;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Erlaubt es die Talentboni zu erzeugen, die es nur bei Professionen gibt.
 *
 * @author Bernhard Jung
 *
 */
class PanelMetaTalentbonusFestProfession extends JPanel {

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Stufenboni gewaehlt
     * werden.
     */
    private static final String BONI =
        I18n.i18n( "PanelMetaTalentbonusFest.Boni" );

    /** Der Text auf dem Knopf mit dem ein Stufenbonus hinzugefuegt wird. */
    private static final String EINFUEGEN =
        I18n.i18n( "PanelMetaTalentbonusFest.Einfuegen" );

    /** Die Ueberschrift fuer die Talente. */
    private static final String TALENTE =
        I18n.i18n( "PanelMetaTalentbonusFest.Talente" );

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaTalentbonusFest.Uebernehmen" );

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner bonus;

    /** Die vom Benutzer, fuer den Bonus, ausgewaehlten Stufenboni. */
    private final JList gewaehlteBoniListe;

    /** Das ListModel fuer die Liste der Stufenboni. */
    private final AenderbaresListModel< Integer > gewaehlteBoniModel;

    /** Die Liste von Talentboni, in die man, ueber dieses Panel, neue Talentboni einfuegt. */
    private final AenderbaresListModel< Talentbonus > talentboni;

    /** Die Combobox zum Auswaehlen der Talentliste. */
    private final JComboBox talentliste;

    /**
     * Der Knopf mit dem man einen im Panel eingestellten Talentbonus in die
     * Liste der Talentboni uebernimmt.
     */
    private final JButton uebernehmen;

    PanelMetaTalentbonusFestProfession(
            final AenderbaresListModel< Talentbonus > dieTalentboni ) {

        bonus = Util.erzeugeSpinner( 0, -9, 99, 1 );

        gewaehlteBoniModel = new AenderbaresListModel< Integer >();

        gewaehlteBoniListe = new JList( gewaehlteBoniModel );
        gewaehlteBoniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        gewaehlteBoniListe.addListSelectionListener(
                new ListSelectionListener() {
                    public void valueChanged( final ListSelectionEvent e ) {
                        if ( e.getValueIsAdjusting() ) {
                            return;
                        }
                        final int index = gewaehlteBoniListe.getSelectedIndex();
                        if ( index >= 0 ) {
                            gewaehlteBoniModel.remove( index );
                            pruefen();
                        }
                    }
                });

        talentboni = dieTalentboni;

        talentliste = new JComboBox( new Object[]{
                TalentListe.FREMDSPRACHEN,
                TalentListe.MUTTERSPRACHE,
                TalentListe.NICHT_MUTTERSPRACHE,
                TalentListe.SCHRIFTEN_MUTTERSPRACHE
        } );

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final TalentbonusFest bonus = new TalentbonusFest(
                        (TalentListe) talentliste.getSelectedItem(),
                        gewaehlteBoniModel.getElemente() );

                talentboni.add( bonus );

                gewaehlteBoniModel.clear();

                pruefen();
            }
        });
        uebernehmen.setEnabled( false );

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JButton einfuegen = new JButton( EINFUEGEN );
        einfuegen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                gewaehlteBoniModel.add( (Integer) bonus.getValue() );
                pruefen();
            }
        });

        final JPanel spinnerPanel = new JPanel(
                new FormLayout( "P, 4dlu, P", "P" ) );

        final CellConstraints cc = new CellConstraints();

        spinnerPanel.add( bonus, cc.xy( 1, 1 ) );

        spinnerPanel.add( einfuegen, cc.xy( 3, 1) );

        final String spalten = "7dlu, F:P:G, 4dlu, F:P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, F:P:G, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        setLayout( layout );

        final DefaultComponentFactory factory =
            DefaultComponentFactory.getInstance();

        add( factory.createSeparator( TALENTE ), cc.xyw( 2, 2, 3 ) );

        add( talentliste, cc.xyw( 2, 4, 3 ) );

        add( factory.createSeparator( BONI ), cc.xyw( 2, 6, 3 ) );

        add( spinnerPanel, cc.xyw( 2, 8, 3 ) );

        add( erzeugeScrollPane( gewaehlteBoniListe, 150, 60 ),
             cc.xyw( 2, 10, 3 ) );

        add( uebernehmen, cc.xyw( 2, 12, 3 ) );
    }

    private JScrollPane erzeugeScrollPane(
            final JComponent component,
            final int breite,
            final int hoehe ) {

        final JScrollPane scrollPane = new JScrollPane( component );

        scrollPane.setPreferredSize( new Dimension( breite, hoehe ) );

        return scrollPane;
    }

    private void pruefen() {

        final int anzahlBoni = gewaehlteBoniModel.getSize();

        uebernehmen.setEnabled( anzahlBoni > 0 );
    }

    /**
     * Loescht die gewaehlten Stufenboni.
     */
    public void reset() {
        gewaehlteBoniModel.clear();
    }
}
