/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilWertBonus;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Beobachtbar;
import de.bernhardjung.dsaprogramm.util.Beobachter;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class PanelMetaVorNachteile
extends JPanel
implements Beobachter, ListSelectionListener {

    /** Die Aufschrift des Knopfes, zum loeschen eines Bonus. */
    private static final String ENTFERNEN =
        I18n.i18n( "PanelMetaVorNachteile.Entfernen" );

    /**
     * Der Reitertext fuer das Tab, auf dem die ausgewaehlten Vor-/Nachteile
     * angezeigt werden.
     */
    private static final String GEWAEHLTE =
        I18n.i18n( "PanelMetaVorNachteile.Gewaehlte" );

    /**
     * Der Text auf dem Knopf, mit dem ein neuer Bonus, in die Liste der Boni
     * uebernommen wird.
     */
    private static final String UEBERNEHMEN =
        I18n.i18n( "PanelMetaVorNachteile.Uebernehmen" );

    /**
     * Der Text fuer die Checkbox, mit der angegeben wird, ob der Benutzer die
     * Variante des gewaehlten Wertes aendern darf.
     */
    private static final String VARIANTE_AENDERBAR_TEXT =
        I18n.i18n( "PanelMetaVorNachteile.VarianteAenderbar.Text" );

    /**
     * Der Hinweistext fuer die Checkbox.
     */
    private static final String VARIANTE_AENDERBAR_TOOLTIP =
        I18n.i18n( "PanelMetaVorNachteile.VarianteAenderbar.Tooltip" );

    /**
     * Eine Liste aller bisher gewaehlten Vor-/Nachteil-Boni.
     */
    private final JList boniListe;

    /** Das ListModel fuer die Liste {@link #boniListe}. */
    private final AenderbaresListModel< VorNachteilBonus > boniModel;

    /**
     * Zeigt den in der Liste ({@link #boniModel}) gewaehlten Bonus an.
     */
    private final JTextArea bonusAnzeige;

    /**
     * Die Werte, die der Benutzer fuer einen neuen Bonus gewaehlt hat.
     */
    private final BeobachtbareListe< VorNachteilWert > liste;

    private final JButton uebernehmen;

    /**
     * Mit dieser Checkbox kann angegeben werden ob der Benutzer die Variante
     * der Werte bei der Erschaffung frei waehlen kann.
     */
    private final JCheckBox varianteAenderbar;

    PanelMetaVorNachteile(
            final Collection< VorNachteil > vorNachteile,
            final String reiterTitel ) {

        boniModel = new AenderbaresListModel< VorNachteilBonus >();

        boniListe = new JList( boniModel );
        boniListe.addListSelectionListener( this );
        boniListe.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        bonusAnzeige = new JTextArea();
        bonusAnzeige.setLineWrap( true );
        bonusAnzeige.setWrapStyleWord( true );
        bonusAnzeige.setEditable( false );

        liste = new BeobachtbareListe< VorNachteilWert >();
        liste.anmelden( this );

        uebernehmen = new JButton( UEBERNEHMEN );
        uebernehmen.setEnabled( false );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                uebernehmen();
            }
        });

        varianteAenderbar = new JCheckBox( VARIANTE_AENDERBAR_TEXT );
        varianteAenderbar.setToolTipText( VARIANTE_AENDERBAR_TOOLTIP );
        varianteAenderbar.setEnabled( false );

        aufbauenBenutzeroberflaeche( getWerte( vorNachteile ), reiterTitel );
    }

    public final void anzeigen( final Collection< VorNachteilBonus > boni ) {

        boniListe.clearSelection();

        boniModel.setElemente( boni );
    }

    private void aufbauenBenutzeroberflaeche(
            final Collection< VorNachteilWert > werte,
            final String reiterTitel ) {

        final JButton entfernen = new JButton( ENTFERNEN );
        entfernen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                entfernen( e );
            }
        });

        final PanelMetaVorNachteileGewaehlte gewaehlte =
            new PanelMetaVorNachteileGewaehlte( liste );

        final PanelMetaVorNachteileWaehlbar waehlbare =
            new PanelMetaVorNachteileWaehlbar( liste );
        waehlbare.setWerte( werte );

        final JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab( GEWAEHLTE, Util.erzeugeScrollPane( gewaehlte ) );

        tabbedPane.addTab( reiterTitel, Util.erzeugeScrollPane( waehlbare ) );

        setLayout( new FormLayout( "7dlu, F:75px:G, 4dlu, F:P:G, 7dlu",
            "7dlu, F:50px:G(0.2), 4dlu, F:100px:G(0.8), 4dlu, P, 4dlu, P, 7dlu" ) );

        final CellConstraints cc = new CellConstraints();

        add( new JScrollPane( bonusAnzeige ), cc.xy( 2, 2 ) );

        add( new JScrollPane( boniListe ), cc.xywh( 2, 4, 1, 3 ) );

        add( entfernen, cc.xy( 2, 8 ) );

        add( tabbedPane, cc.xywh( 4, 2, 1, 3 ) );

        add( varianteAenderbar, cc.xy( 4, 6 ) );

        add( uebernehmen, cc.xy( 4, 8 ) );
    }

    /**
     * Wird aufgerufen, wenn der gewaehlte Bonus aus der Liste
     * {@link #boniModel} entfernt werden soll.
     */
    private final void entfernen( final ActionEvent e ) {

        final int index = boniListe.getSelectedIndex();

        if ( index >= 0 ) {

            boniModel.remove( index );
        }
    }

    /**
     * @return eine Liste der Vor-/Nachteil-Boni.
     */
    public List< VorNachteilBonus > getBoni() {
        return boniModel.getElemente();
    }

    public Collection< VorNachteilWert > getWerte(
            final Collection< VorNachteil > vorNachteile ) {

        final Collection< VorNachteilWert > werte =
            new ArrayList< VorNachteilWert >( vorNachteile.size() );

        for ( final VorNachteil vorNachteil : vorNachteile ) {

            final VorNachteilWert wert;

            if ( vorNachteil.hatVarianten() ) {

                final List< Object > varianten = vorNachteil.getVarianten( null );

                final Object variante;

                if ( varianten.size() > 0 ) {

                    variante = varianten.get( 0 );

                } else {

                    variante = "";
                }

                wert = new VorNachteilWert( vorNachteil, variante );

            } else {

                wert = new VorNachteilWert( vorNachteil );
            }

            werte.add( wert );
        }

        return werte;
    }

    public void melden( final Beobachtbar b, final Object arg ) {

        uebernehmen.setEnabled( liste.size() > 0 );

        boolean hatVarianten = false;

        for ( final VorNachteilWert wert : liste.getElemente() ) {

            if ( wert.getVorNachteil().hatVarianten() ) {

                hatVarianten = true;

                break;
            }
        }

        if ( hatVarianten ) {

            varianteAenderbar.setEnabled( true );

        } else {

            varianteAenderbar.setEnabled( false );
            varianteAenderbar.setSelected( false );
        }
    }

    /**
     * Loescht die Liste der Vor-/Nachteile und alle Eintraege in den Paneln.
     */
    public void reset() {
        boniModel.clear();
        liste.clear();
    }

    private void uebernehmen() {

        final List< VorNachteilWertBonus > wertBoni =
            new ArrayList< VorNachteilWertBonus >();

        for ( final VorNachteilWert wert : liste.getElemente() ) {

            if ( wert.getVorNachteil().hatVarianten() ) {

                final String id = VorNachteil.ermitteleVariantenId( wert );

                wertBoni.add( new VorNachteilWertBonus( wert, id ) );

            } else {

                wertBoni.add( new VorNachteilWertBonus( wert ) );

            }
        }

        final VorNachteilBonus bonus =
            new VorNachteilBonus( wertBoni, varianteAenderbar.isSelected() );

        boniModel.add( bonus );

        liste.clear();
    }

    /**
     * Wird aufgerufen wenn in der Liste {@link #boniModel} ein Talentbonus
     * gewaehlt wird.
     *
     * @param e
     */
    public void valueChanged( final ListSelectionEvent e ) {

        if ( e.getValueIsAdjusting() ) {

            return;
        }

        final Object gewaehlt = boniListe.getSelectedValue();

        if ( null != gewaehlt ) {

            bonusAnzeige.setText( gewaehlt.toString() );

        } else {

            bonusAnzeige.setText( "" );
        }
    }

}
