/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.FernkampfwaffeWert;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.NahkampfwaffeWert;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

class PanelAusruestung extends JPanel {

    private static final String FERNKAMPF =
        I18n.i18n( "PanelAusruestung.Fernkampf" );

    private static final String FERNKAMPF_REITER =
        I18n.i18n( "PanelAusruestung.Fernkampf.Reiter" );

    private static final String NAHKAMPF =
        I18n.i18n( "PanelAusruestung.Nahkampf" );

    private static final String NAHKAMPF_REITER =
        I18n.i18n( "PanelAusruestung.Nahkampf.Reiter" );

    private static final String RUESTUNG =
        I18n.i18n( "PanelAusruestung.Ruestung" );

    private static final String RUESTUNG_REITER =
        I18n.i18n( "PanelAusruestung.Ruestung.Reiter" );

    private static final String SCHILDE =
        I18n.i18n( "PanelAusruestung.Schilde" );

    private static final String SCHILDE_REITER =
        I18n.i18n( "PanelAusruestung.Schilde.Reiter" );

    private final List< PanelFernkampfwaffe > fernkampfwaffen;

    private final List< PanelNahkampfwaffe > nahkampfwaffen;

    private final PanelRuestungen ruestungen;

    private final PanelSchilde schilde;

    public PanelAusruestung() {

        fernkampfwaffen = new ArrayList< PanelFernkampfwaffe >( 3 );
        for ( int i = 0 ; i < Held.ANZAHL_FERNKAMPFWAFFEN ; ++i ) {
            fernkampfwaffen.add( new PanelFernkampfwaffe() );
        }

        nahkampfwaffen = new ArrayList< PanelNahkampfwaffe >( 5 );
        for ( int i = 0 ; i < Held.ANZAHL_NAHKAMPFWAFFEN ; ++i ) {
            nahkampfwaffen.add( new PanelNahkampfwaffe() );
        }

        ruestungen = new PanelRuestungen();

        schilde = new PanelSchilde();

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        tabbedPane.addTab( NAHKAMPF_REITER,
                           erzeugeFrame(
                                   NAHKAMPF,
                                   erzeugePanel( nahkampfwaffen ) ) );

        tabbedPane.addTab( FERNKAMPF_REITER,
                           erzeugeFrame(
                                   FERNKAMPF,
                                   erzeugePanel( fernkampfwaffen ) ) );

        tabbedPane.addTab( RUESTUNG_REITER,
                           erzeugeFrame( RUESTUNG, ruestungen ) );

        tabbedPane.addTab( SCHILDE_REITER,
                           erzeugeFrame( SCHILDE, schilde ) );

        setLayout( new BorderLayout() );

        add( tabbedPane, BorderLayout.CENTER );
    }

    private SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JPanel inhalt ) {

        final SimpleInternalFrame frame =
            new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( Util.erzeugeScrollPane( inhalt ) );

        return frame;
    }

    /**
     * Erzeugt ein Panel, das mehrere Panel vertikal anordnet.
     *
     * @param panels
     *            die untereinander angeordnet werden sollen.
     * @return ein neu erzeugtes Panel.
     */
    private JPanel erzeugePanel(
            final List< ? extends JPanel > panels ) {

        final FormLayout layout = new FormLayout( "P:G" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        for ( final JPanel panel : panels ) {

            builder.append( panel );
        }

        return builder.getPanel();
    }

    /**
     * Bestimmt den Held der ueber das Panel ausgeruestet wird.
     *
     * @param held
     *            der ausgeruestet werden soll.
     */
    public final void setHeld( final Held held ) {

        final List< FernkampfwaffeWert > fern = held.getFernkampfwaffen();

        for ( int i = 0, n = fernkampfwaffen.size(); i < n ; ++i ) {

            FernkampfwaffeWert fernkampfwaffe = null;

            if ( fern.size() > i ) {

                fernkampfwaffe = fern.get( i );
            }

            fernkampfwaffen.get( i ).setHeld( held, fernkampfwaffe );
        }

        final List< NahkampfwaffeWert > nah = held.getNahkampfwaffen();

        for ( int i = 0, n = nahkampfwaffen.size() ; i < n ; ++i ) {

            NahkampfwaffeWert nahkampfwaffe = null;

            if ( nah.size() > i ) {

                nahkampfwaffe = nah.get( i );
            }

            nahkampfwaffen.get( i ).setHeld( held, nahkampfwaffe );
        }

        ruestungen.setHeld( held );

        schilde.setHeld( held );
    }
}
