/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import static de.bernhardjung.dsaprogramm.basis.Held.ANZAHL_RUESTUNGSTEILE;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.basis.RuestungWert;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

class PanelRuestungen extends JPanel {

    /** Die Ueberschrift fuer die Spalte mit der Behinderung. */
    private static final String BE = I18n.i18n( "PanelRuestungen.Be" );

    /** Die Ueberschrift fuer die Spalte mit dem Namen der Ruestung. */
    private static final String NAME = I18n.i18n( "PanelRuestungen.Name" );

    /** Die Ueberschrift fuer die Spalte mit dem Ruestungsschutz. */
    private static final String RS = I18n.i18n( "PanelRuestungen.Rs" );

    /**
     * Die Ueberschrift fuer die Spalte mit den Comboboxem mit denen man die
     * Ruestung waehlen kann.
     */
    private static final String RUESTUNG = I18n.i18n( "PanelRuestungen.Ruestung" );

    /** Die Spinner fuer die Behinderung. */
    private final List< JSpinner > be;

    /**
     * Die Checkboxen mit denen eine Ruestung fuer den Helden (ab)gewaehlt
     * werden kann.
     */
    private final List< JCheckBox > gewaehlt;

    /** Der Held fuer den die Ruestungen gewaehlt werden. */
    private Held held;

    /** Die Textfelder fuer die Namen der Ruestungen. */
    private final List< JTextField > name;

    /** Die Spinner fuer den Ruestungsschutz. */
    private final List< JSpinner > rs;

    /** Die Ruestungen, die ueber das Panel editiert werden. */
    private final List< RuestungWert > ruestungWert;

    /** Die Comboboxen mit denen man einen Ruestungstyp auswaehlt. */
    private final List< JComboBox > ruestung;

    /** Der ActionListener fuer die Comboboxen in {@link #ruestung} */
    private final ActionListener ruestungListener;

    public PanelRuestungen() {

        final ChangeListener beListener = new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                beGeaendert( e );
            }
        };

        be = new ArrayList< JSpinner >( ANZAHL_RUESTUNGSTEILE );
        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );

            spinner.setName( String.valueOf( i ) );

            spinner.addChangeListener( beListener );

            be.add( spinner );
        }

        final ActionListener gewaehltListener = new ActionListener(){
            public void actionPerformed( ActionEvent e ) {
                anAbgewaehlt( e );
            }
        };

        gewaehlt = new ArrayList< JCheckBox >( ANZAHL_RUESTUNGSTEILE );
        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            final JCheckBox checkBox = new JCheckBox();

            checkBox.setName( String.valueOf( i ) );

            checkBox.addActionListener( gewaehltListener );

            gewaehlt.add( checkBox );
        }

        name = new ArrayList< JTextField >( ANZAHL_RUESTUNGSTEILE );
        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            final int index = i;

            final JTextField textfeld = new JTextField();

            textfeld.getDocument().addDocumentListener( new DocumentListener() {
                public void changedUpdate( final DocumentEvent e ) {}

                public void insertUpdate( final DocumentEvent e ) {
                    setName();
                }

                public void removeUpdate( final DocumentEvent e ) {
                    setName();
                }

                private void setName() {

                    final RuestungWert ruestung = ruestungWert.get( index );

                    final String text = name.get( index ).getText();

                    ruestung.setName( text );
                }
            });

            name.add( textfeld );
        }


        final ChangeListener rsListener = new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                rsGeaendert( e );
            }
        };

        rs = new ArrayList< JSpinner >( ANZAHL_RUESTUNGSTEILE );
        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );

            spinner.setName( String.valueOf( i ) );

            spinner.addChangeListener( rsListener );

            rs.add( spinner );
        }

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        final Vector< Ruestung > vector =
            new Vector< Ruestung >( speicher.getRuestungen() );

        ruestungListener = new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                ruestungGewaehlt( e );
            }
        };

        ruestung = new ArrayList< JComboBox >( ANZAHL_RUESTUNGSTEILE );
        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            final JComboBox comboBox = new JComboBox( vector );

            comboBox.setName( String.valueOf( i ) );

            comboBox.addActionListener( ruestungListener );

            ruestung.add( comboBox );
        }

        ruestungWert = new ArrayList< RuestungWert >( ANZAHL_RUESTUNGSTEILE );
        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            ruestungWert.add( new RuestungWert( vector.get( 0 ) ) );

            aktualisiereAnzeige( i );
        }

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Wird aufgerufen, wenn eine Ruestung an-/abgewaehlt wird. Fuegt dem Held
     * die Ruestung hinzu, bzw. entfernt sie von ihm.
     *
     * @param e
     *            der ActionEvent der beim Klick auf die Checkbox ausgeloest
     *            wurde.
     */
    private void anAbgewaehlt( final ActionEvent e ) {

        final JCheckBox checkBox = (JCheckBox) e.getSource();

        final int index = Integer.valueOf( checkBox.getName() );

        final RuestungWert ruestung = ruestungWert.get( index );

        if ( checkBox.isSelected() ) {

            held.einfuegen( ruestung );

        } else {

            held.entfernen( ruestung );

        }
    }

    /**
     * Uebernimmt die Werte aus einem Wert-Objekt in die
     * Benutzeroberflaeche.
     *
     * @param index
     *            der Ruestung in {@link #ruestungWert}. Die Werte werden in Zeile
     *            {@code index} in der Benutzeroberflaeche angezeigt.
     */
    private void aktualisiereAnzeige( final int index ) {

        final RuestungWert r = ruestungWert.get( index );

        be.get( index ).setValue( r.getBe() );

        name.get( index ).setText( r.getName() );

        rs.get( index ).setValue( r.getRs() );

        ruestung.get( index ).removeActionListener( ruestungListener );
        ruestung.get( index ).setSelectedItem( r.getRuestung() );
        ruestung.get( index ).addActionListener( ruestungListener );
    }

    private void aufbauenBenutzeroberflaeche() {

        final String spalten = "P, 4dlu, 100px:G, 4dlu, P, 4dlu, P, 4dlu, P";

        final FormLayout layout = new FormLayout( spalten );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        builder.append( "" );
        builder.append( Util.erzeugeUeberschrift( NAME ) );
        builder.append( Util.erzeugeUeberschrift( RUESTUNG  ) );
        builder.append( Util.erzeugeUeberschrift( RS   ) );
        builder.append( Util.erzeugeUeberschrift( BE   ) );

        for ( int i = 0 ; i < ANZAHL_RUESTUNGSTEILE ; ++i ) {

            builder.append( gewaehlt.get( i ) );
            builder.append( name.get( i ) );
            builder.append( ruestung.get( i ) );
            builder.append( rs.get( i ) );
            builder.append( be.get( i ) );
        }
    }

    /**
     * Wird aufgerufen, wenn ein Spinner fuer die Behinderung veraendert wird.
     * Aktualisiert die Behinderung.
     *
     * @param e
     *            der Changeevent, der vom Spinner ausgeloest wurde.
     */
    private void beGeaendert( final ChangeEvent e ) {

        final JSpinner spinner = (JSpinner) e.getSource();

        final int index = Integer.valueOf( spinner.getName() );

        final RuestungWert ruestung = ruestungWert.get( index );

        final int be = (Integer) spinner.getValue();

        ruestung.setBe( be );
    }

    /**
     * Wird aufgerufen, wenn ein Spinner fuer den Ruestungsschutz veraendert
     * wird. Aktualisiert den Ruestungsschutz.
     *
     * @param e
     *            der Changeevent, der vom Spinner augeloest wurde.
     */
    private void rsGeaendert( final ChangeEvent e ) {

        final JSpinner spinner = (JSpinner) e.getSource();

        final int index = Integer.valueOf( spinner.getName() );

        final RuestungWert ruestung = ruestungWert.get( index );

        final int rs = (Integer) spinner.getValue();

        ruestung.setRs( rs );
    }

    /**
     * Legt den Held fest, fuer den die Ruestungen gewaehlt werden.
     *
     * @param derHeld
     *            fuer den dir Ruestung gewaehlt wird.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        final List< RuestungWert > heldenruestungen = held.getRuestungen();

        for ( final JCheckBox checkBox : gewaehlt ) {

            checkBox.setSelected( false );
        }

        for ( int i = 0, n = heldenruestungen.size() ; i < n ; ++i ) {

            ruestungWert.set( i, heldenruestungen.get( i ) );

            gewaehlt.get( i ).setSelected( true );

            aktualisiereAnzeige( i );
        }
    }

    /**
     * Wird aufgerufen, wenn mit einer Combobox einer Ruestung gewaehlt wird.
     * Aktualisert die Anzeige entsprechend der Ruestung. Uebernimmt die Werte
     * der Ruestung in die Benutzeroberflaeche.
     *
     * @param e
     *            der Actionevent, der von der Combobox ausgeloest wurde.
     */
    private void ruestungGewaehlt( final ActionEvent e ) {

        final JComboBox comboBox = (JComboBox) e.getSource();

        final int index = Integer.valueOf( comboBox.getName() );

        final RuestungWert ruestung = ruestungWert.get( index );

        final Ruestung r = (Ruestung) comboBox.getSelectedItem();

        ruestung.setRuestung( r );

        aktualisiereAnzeige( index );
    }
}
