/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.gui.Lernmethoden;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class PanelVorNachteileSteigern extends JPanel {

    private static final String LERNMETHODE =
        I18n.i18n( "PanelVorNachteileSteigern.Lernmethode" );

    private static final String NACHTEILE =
        I18n.i18n( "PanelVorNachteileSteigern.Nachteile" );

    private static final String VOR_NACHTEILE =
        I18n.i18n( "PanelVorNachteileSteigern.VorNachteile" );

    private static final String VORTEILE =
        I18n.i18n( "PanelVorNachteileSteigern.Vorteile" );

    /** Die Liste der Vor-/Nachteile die der Held besitzt. */
    private final PanelVorNachteileListeSteigern gewaehlte;

    /** Die Liste der Nachteile. */
    private final PanelVorNachteileListeSteigern nachteile;

    /** Die Liste der Vorteile. */
    private final PanelVorNachteileListeSteigern vorteile;

    PanelVorNachteileSteigern() {

        final Vector< Lernmethode > lern = new Vector< Lernmethode >();
        lern.add( Lernmethode.LEHRMEISTER );
        lern.add( Lernmethode.SPEZIELLE_ERFAHRUNG );
        lern.add( Lernmethode.SELBSTSTUDIUM );
        final JComboBox comboBoxLernmethoden = new JComboBox( lern );

        final Lernmethoden lernmethoden =
            new Lernmethoden( comboBoxLernmethoden );

        gewaehlte = new PanelVorNachteileListeSteigern(
                true,
                VOR_NACHTEILE,
                lernmethoden );

        nachteile = new PanelVorNachteileListeSteigern(
                false,
                NACHTEILE,
                lernmethoden );

        List< VorNachteilWert > werte =
            erzeugeWerte( VorNachteil.getNachteile() );

        nachteile.setWerte( werte );

        vorteile = new PanelVorNachteileListeSteigern(
                false,
                VORTEILE,
                lernmethoden );

        werte = erzeugeWerte( VorNachteil.getVorteile() );

        vorteile.setWerte( werte );


        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        tabbedPane.addTab(
                I18n.i18n( "PanelVorNachteileSteigern.Reiter.Gewaehlte" ),
                erzeugeFrame( I18n.i18n( "PanelVorNachteileSteigern.Gewaehlte"),
                              gewaehlte ) );

        tabbedPane.addTab(
                I18n.i18n( "PanelVorNachteileSteigern.Reiter.Vorteile" ),
                erzeugeFrame( VORTEILE, vorteile ) );

        tabbedPane.addTab(
                I18n.i18n( "PanelVorNachteileSteigern.Reiter.Nachteile" ),
                erzeugeFrame( NACHTEILE, nachteile ) );

        setLayout( new FormLayout( 
                "P:G, R:P, 2dlu, L:P, P:G", 
                "2dlu, P, 2dlu, F:D:G" ) );

        final CellConstraints cc = new CellConstraints();

        add( new JLabel( LERNMETHODE ), cc.xy( 2, 2 ) );

        add( comboBoxLernmethoden, cc.xy( 4, 2 ) );

        add( tabbedPane, cc.xyw( 1, 4, 5 ) );
    }

    /**
     * Packt ein Panel in einen {@link SimpleInternalFrame} und gibt ihm eine
     * Ueberschrift.
     *
     * @param titel Die Ueberschrift des Frames.
     * @param inhalt des Frames.
     * @return der Frame mit angegebenem Inhalt und Titel.
     */
    private final SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JPanel inhalt ) {

        final SimpleInternalFrame frame;
        frame = new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( inhalt );

        return frame;
    }

    /**
     * Erzeugt zu einer List an Vor-/Nachteilen die entsprechenden Werte.
     *
     * @param vorNachteile
     *            zu denen die Werte erzeugt werden sollen.
     * @return eine Liste von VorNachteilWerten.
     */
    private final List< VorNachteilWert > erzeugeWerte(
            final List< VorNachteil > vorNachteile ) {

        final List< VorNachteilWert > werte =
            new ArrayList< VorNachteilWert >( vorNachteile.size() );

        for ( final VorNachteil vorNachteil : vorNachteile ) {

            if ( vorNachteil.hatVarianten() ) {

                final List< Object > varianten = vorNachteil.getVarianten(null);

                final Object variante;
                if ( 0 < varianten.size() ) {

                    variante = varianten.get( 0 );

                } else {

                    variante = "";

                }

                werte.add( new VorNachteilWert( vorNachteil, variante ) );

            } else {

                werte.add( new VorNachteilWert( vorNachteil ) );

            }
        }

        return werte;
    }

    /**
     * @param held dessen Werte angezeigt und manipuliert werden.
     */
    public final void setHeld( final Held held ) {

        gewaehlte.setHeld( held );

        nachteile.setHeld( held );

        vorteile.setHeld( held );

    }

}
