/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Halbiert die Kosten, wenn der Held den Vorteil Schlangenmensch besitzt.
 *
 * @author Bernhard Jung
 *
 */
class KostenSchlangenmensch extends KostenKampf {

    KostenSchlangenmensch( final int apKosten ) {
        super( apKosten );
    }

    @Override
    public Collection< Object > getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();

        abhaengigkeiten.add( VorNachteil.SCHLANGENMENSCH );

        return abhaengigkeiten;
    }

    /**
     * Halbiert die Kosten, wenn der Held den Vorteil Schlangenmensch besitzt.
     */
    @Override
    protected float getFaktor(
            final Held held,
            final SonderfertigkeitWert wert ) {

        float faktor = super.getFaktor( held, wert );

        if ( held.hat( VorNachteil.SCHLANGENMENSCH )
          && ( faktor > 0.5 ) ) {

            // Diese Verbilligung soll nicht zusaetzlich zur normalen
            // Verbilligung wirken, daher wird sie nur angewendet, wenn der
            // Faktor noch ueber 0.5 ist.
            // Damit wirkt sie noch zusaetzlich zur Verbilligung durch
            // Akademische Ausbildung (Krieger).

            faktor *= 0.5F;
        }

        return faktor;
    }
}
