/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class KostenSchriftWissen extends Kosten {

    private VorNachteilWert begabungTalent;

    private VorNachteilWert begabungTalentgruppe;

    /**
     * @return Akad. Ausb. Gelehrter, Begabung Talent(gruppe), Elfische
     *         Weltsicht, Unfaehigkeit Talent(gruppe).
     */
    @Override
    public Collection<Object> getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();

        abhaengigkeiten.add( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENT );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENTGRUPPE );

        abhaengigkeiten.add( VorNachteil.ELFISCHE_WELTSICHT );

        return abhaengigkeiten;
    }

    /** {@inheritDoc} */
    @Override
    public int getSteigerungsspalte(
            final Held held,
            final Talent talent,
            final int stufe, Lernmethode lernmethode ) {

        int steigerungsfaktor = super.getSteigerungsspalte(held, talent, stufe, lernmethode);

        setBegabung( talent );

        if ( held.hat( begabungTalent ) && held.hat( begabungTalentgruppe ) ) {

            steigerungsfaktor -= 2;

        } else if ( held.hat( begabungTalent )
                 || held.hat( begabungTalentgruppe )
                 || ( held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_GELEHRTER )
                     && ( stufe < 10 )
                     && held.wirdErschaffen() ) ) {

            --steigerungsfaktor;
        }

        if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
          && ( ! held.hatLeittalent( talent ) ) ) {

            ++steigerungsfaktor;
        }

        return steigerungsfaktor;
    }

    /**
     * Initialisiert die Begabungen und Unfaehigkeiten und setzt die Varianten
     * entsprechend dem uebergebenen Talent.
     *
     * @param talent
     *            fuer die Begabungen und Unfaehigkeiten.
     */
    private void setBegabung( final Talent talent ) {

        if ( null == begabungTalent ) {

            begabungTalent =
                new VorNachteilWert( VorNachteil.BEGABUNG_TALENT,
                                     talent );

            begabungTalentgruppe =
                new VorNachteilWert( VorNachteil.BEGABUNG_TALENTGRUPPE,
                                     talent.getTalentgruppe() );

        } else {

            begabungTalent.setVariante( talent );

            begabungTalentgruppe.setVariante( talent.getTalentgruppe() );

        }
    }
}
