/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.stufen;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;

/**
 * Ein Objekt das sich um die Berechnung der Modifikation und der Hoechststufe 
 * fuer ein Talent kuemmert. 
 * Fuer jede Implementierung des Interface existiert eine statische Konstante
 * in diesem Interface. 
 * 
 * @author Bernhard Jung
 */
public interface Stufen {

    /**
     * Die Implementierung die fuer die meisten Talente, mit Ausnahme von 
     * Sprachen und Schriften, gilt.
     */
    public static final Stufen TALENT = new StufenTalent();
    
    /** Die Implementierung des Interface fuer die Sprachen. */
    public static final Stufen SPRACHE = new StufenSprache();
    
    /** Die Implementierung des Interface fuer die Schriften. */
    public static final Stufen SCHRIFT = new StufenSchrift();
    
    /**
     * @return die Regelelement von denen die Berechnung der Stufe abhaengig 
     *         ist.
     */
    public Collection<Object> getAbhaengigkeiten();
    
    /**
     * Liefert die Hoechststufe bis zu der eine Talent gesteigert werden kann.
     * 
     * @param held der das Talent steigern will.
     * @param talent das gesteigert werden soll.
     * @return die hoechste noch erlaubte Stufe in dem Talent.
     */
    public int getHoechststufe( final Held held, final Talent talent );

    /**
     * Liefert die Modifikation die ein Helden zB durch Vor-/Nachteile
     * auf die Stufe eines Talentes erhaelt.
     * 
     * @param held der das Talent besitzt.
     * @param talent dessen Modifikation gesucht ist.
     * @return die Modifikation die der Held auf die Stufe des Talents
     *      erhaelt.
     */
    public int getModifikation( final Held held, final Talent talent );
    
}
