/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.stufen;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Schrift;
import de.bernhardjung.dsaprogramm.talente.Talent;

/**
 * Bestimmt die Hoechststufe einer Schrift an Hand der Komplexitaet der 
 * Schrift.
 */
class StufenSchrift extends StufenTalent {

    /**
     * Liefert die Hoechststufe bis zu der eine Schrift gesteigert werden kann.
     * 
     * @param held der die Schrift steigern will.
     * @param talent die Schrift deren Hoechststufe gesucht ist.
     * @return die hoechste noch erlaubte Stufe die {@code held} in der Schrift
     *      besitzen darf.
     */
    @Override
    public int getHoechststufe( final Held held, final Talent talent ) {
        
        // Die Hoechststufe durch die Eigenschaften
        final int hoechststufe = super.getHoechststufe( held, talent );
        
        final int komplexitaet = ((Schrift) talent).getKomplexitaet(); 
        
        return Math.min( hoechststufe, komplexitaet );
    }
}
