/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Prueft ob ein Held eine bestimmte Rasse besitzt. Nur diese Rasse darf den
 * Vor-/Nachteil frei waehlen.
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungRasse extends Voraussetzung {

    private static final String NUR_FUER = I18n.i18n( "VoraussetzungRasse" );

    /** Die Rasse die den Vor-/Nachteil frei waehlen darf. */
    private final Rasse rasse;

    /**
     * @param rasseId
     *            die Id der Rasse die den Vor-/Nachteil frei waehlen darf.
     */
    VoraussetzungRasse( final String rasseId ) {

        rasse = Datenspeicher.getInstanz().getRasse( rasseId );
    }

    /** @return eine leere Liste. */
    @Override
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /**
     * Prueft ob der Held eine bestimmte Rasse besitzt und falls nicht wir eine
     * Konfliktbeschreibung ausgegeben.
     */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( ( ! held.hatVorgegeben( wert ) )
          && ( ! held.getRasse().equals( rasse ) ) ) {

            beschreibung.einfuegen( String.format(
                    NUR_FUER, rasse.getName() ) );
        }

        return beschreibung;
    }

}
