/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

/**
 * Ueberprueft ob der Held den entsprechenden Vorteil zu dem Zauber besitzt.
 *
 * @author Bernhard Jung
 *
 */
public class VoraussetzungBegabung implements Voraussetzung {

    /** Erklaert dem Benutzer, dass ein Vorteil benoetigt wird. */
    private static final String BENOETIGT = VoraussetzungVorteil.BENOETIGT;

    /** Der Vorteil Uebernatuerliche Begabung. */
    private VorNachteilWert begabung;

    /** @return den Vorteil Uebernatuerliche Begabung */
    public final Collection< Object > getAbhaengigkeiten() {
        return Arrays.asList( (Object)
                VorNachteil.getVorNachteil(
                        "VorNachteil.UebernatuerlicheBegabung" ) );
    }

    /** {@inheritDoc} */
    public final KonfliktBeschreibung pruefen(
            final Held held,
            final ZauberWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        initWert( wert.getZauber() );

        if ( ! held.hat( begabung ) ) {

            beschreibung.einfuegen( String.format( BENOETIGT, begabung.getName() ) );
        }

        return beschreibung;
    }

    /** Initialisiert die {@link #begabung} mit dem uebergebenen Zauber. */
    private final void initWert( final Zauber zauber ) {

        if ( null == begabung ) {

            begabung = new VorNachteilWert(
                    VorNachteil.getVorNachteil(
                            "VorNachteil.UebernatuerlicheBegabung" ),
                    zauber );

        } else {

            begabung.setVariante( zauber );
        }
    }
}
